import unittest

import mock
from mock import MagicMock, Mock

from savory_pie.django.validators import ValidationError
from savory_pie.django.resources import ModelResource
from savory_pie.fields import AttributeField, IterableField, SubObjectResourceField, CompleteURIResourceField
from savory_pie.tests.mock_context import mock_context


class IterableFieldTest(unittest.TestCase):

    def test_handle_outgoing_multi_level(self):

        # TODO: Need to alter savory_pie.fields.IterableField.handle_outgoing to not use manager.all()
        from savory_pie.tests.django import mock_orm

        class ORMModelMock(mock_orm.Model):
            pass

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = IterableField(attribute='foo.fu', resource_class=MockResource)

        source_object = mock_orm.Model()
        related_manager = mock_orm.Manager()
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            ORMModelMock(pk=4, bar=14)
        ))

        class InterimObject(object):
            pass

        source_object.foo = InterimObject()
        source_object.foo.fu = related_manager

        target_dict = {}
        field.handle_outgoing(mock_context(), source_object, target_dict)
        self.assertEqual([{'_id': '4', 'bar': 14}], target_dict['foo.fu'])

    def test_iterable_factory_outgoing(self):
        values = [
            Mock(name='value1', bar=1),
            Mock(name='value2', bar=2),
            Mock(name='value3', bar=3),
        ]
        iterable = MagicMock(name='iterable')
        iterable.__iter__.return_value = iter(values)

        class MockResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        iterable_factory = Mock(name='iterable_factory', return_value=iterable)

        source_object = Mock(name='source_object')
        target_dict = {}

        field = IterableField(attribute='foo', resource_class=MockResource, iterable_factory=iterable_factory)

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(
            target_dict['foo'],
            [
                {
                    'bar': 1,
                    '_id': str(values[0].pk),
                },
                {
                    'bar': 2,
                    '_id': str(values[1].pk),
                },
                {
                    'bar': 3,
                    '_id': str(values[2].pk),
                },
            ]
        )

        iterable_factory.assert_called_with(source_object.foo)

    def test_incoming_push_pop(self):
        Resource = Mock(name='resource')
        ctx = mock_context()

        field = IterableField(attribute='foo', resource_class=Resource)

        source_dict = {
            'foo': {'bar': 20},
        }

        target_object = MagicMock()
        target_object.__iter__ = iter([])
        field.handle_incoming(ctx, source_dict, target_object)

        ctx.assert_has_calls([
            mock.call.push(target_object),
            mock.call.pop(),
        ])


class IterableFieldTestCase(unittest.TestCase):
    def test_handle_incoming_pre_save_optional(self):
        ctx = mock_context()
        ctx.resolve_resource_uri = Mock(name='resolve_resource_uri')
        sub_resource = ctx.resolve_resource_uri.return_value
        Resource = Mock(name='resource')

        source_dict = {
            'foo': {
                'resourceUri': 'foo'
            },
        }
        target_obj = Mock(['save', 'pre_save'], name='target_obj',)
        field = SubObjectResourceField('foo', Resource)

        field.handle_incoming(ctx, source_dict, target_obj)

        self.assertEqual(sub_resource.model, target_obj.foo)

    def test_incoming_push_pop(self):
        Resource = Mock(name='resource')
        ctx = mock_context()

        field = SubObjectResourceField(attribute='foo', resource_class=Resource)

        source_dict = {
            'foo': {'bar': 20},
        }

        target_object = Mock()
        field.handle_incoming(ctx, source_dict, target_object)

        ctx.assert_has_calls([
            mock.call.push(target_object),
            mock.call.pop(),
        ])


class CompleteURIResourceFieldTestCase(unittest.TestCase):
    def test_outgoing(self):

        class Resource(ModelResource):
            model_class = Mock()
            parent_resource_path = 'resources'

        field = CompleteURIResourceField(resource_class=Resource)

        source_object = Mock()
        ctx = mock_context()
        ctx.build_resource_uri = Mock()
        ctx.build_resource_uri.side_effect = ['uri://resources/1']

        target_dict = dict()
        field.handle_outgoing(ctx, source_object, target_dict)

        self.assertEqual(target_dict['completeResourceUri'], 'uri://resources/1')


class AttributeFieldTestCase(unittest.TestCase):
    def test_incoming_required_and_present(self):
        ctx = mock_context()
        source_dict = {
            'foo': 20
        }

        class TargetObject:
            foo = 0
        target_object = TargetObject()
        field = AttributeField('foo', type=int)

        field.handle_incoming(ctx, source_dict, target_object)
        self.assertEqual(target_object.foo, 20)

    def test_incoming_required_and_missing(self):
        ctx = mock_context()
        source_dict = {
            'foo': 20
        }

        class TargetObject:
            foo = 0
        target_object = TargetObject()
        field = AttributeField('bar', type=int)

        with self.assertRaises(ValidationError):
            field.handle_incoming(ctx, source_dict, target_object)
        self.assertEqual(target_object.foo, 0)

    def test_incoming_optional_and_present(self):
        ctx = mock_context()
        source_dict = {
            'foo': 20
        }

        class TargetObject:
            foo = 0
        target_object = TargetObject()
        field = AttributeField('foo', type=int, optional=True)

        field.handle_incoming(ctx, source_dict, target_object)
        self.assertEqual(target_object.foo, 20)

    def test_incoming_optional_and_missing(self):
        ctx = mock_context()
        source_dict = {
            'foo': 20
        }

        class TargetObject:
            foo = 0
        target_object = TargetObject()
        field = AttributeField('bar', type=int, optional=True)

        field.handle_incoming(ctx, source_dict, target_object)
        self.assertEqual(target_object.foo, 0)



import unittest
import mock
from savory_pie.resources import _ParamsImpl, EmptyParams


class EmptyParamsTestCase(unittest.TestCase):

    def test_get(self):
        params = EmptyParams()
        self.assertEqual(params.get('key', default='value'), 'value')

    def test_get_as(self):
        params = EmptyParams()
        self.assertEqual(params.get_as('key', str, default='value'), 'value')

    def test_get_list(self):
        params = EmptyParams()
        self.assertEqual(params.get_list('key'), [])

    def test_get_list_of(self):
        params = EmptyParams()
        self.assertEqual(params.get_list_of('key', str), [])


class ParamsImplTestCase(unittest.TestCase):
    def test_init(self):
        params = _ParamsImpl('some var')
        self.assertEqual(params._GET, 'some var')

    def test_keys(self):
        params = _ParamsImpl({'key1': 'value1', 'key2': 'value2'})
        self.assertEqual(sorted(params.keys()), ['key1', 'key2'])

    def test_contains(self):
        params = _ParamsImpl({'key1': 'value1', 'key2': 'value2'})
        self.assertTrue('key1' in params)
        self.assertFalse('key3' in params)

    def test_get_item(self):
        params = _ParamsImpl({'key1': 'value1', 'key2': 'value2'})
        self.assertEqual(params['key1'], 'value1')

    def test_parameters_get(self):
        params = _ParamsImpl({'key1': 'value1'})
        self.assertEqual(params.get('key1', 'other'), 'value1')

    def test_parameters_get_default(self):
        params = _ParamsImpl({'key1': 'value1'})
        self.assertEqual(params.get('some', 'other'), 'other')

    def test_parameters_get_as(self):
        params = _ParamsImpl({'key1': 'value1'})
        self.assertEqual(params.get_as('key1', str, 'other'), 'value1')

    def test_parameters_get_as_wrong_type(self):
        params = _ParamsImpl({'key1': 1})
        self.assertEqual(params.get_as('key1', str, 'other'), '1')

    def test_parameters_get_as_default(self):
        params = _ParamsImpl({'key1': 1})
        self.assertEqual(params.get_as('key2', str, 'other'), 'other')

    def test_parameters_get_list(self):
        get = mock.Mock()
        get.getlist.return_value = ['value1']
        params = _ParamsImpl(get)
        self.assertEqual(params.get_list('key1'), ['value1'])

    def test_get_list_of(self):
        params = _ParamsImpl({'key1': [1]})
        self.assertEqual(params.get_list_of('key1', str), ['1'])

    def test_get_list_of_not_found(self):
        params = _ParamsImpl({'key1': [1]})
        self.assertEqual(params.get_list_of('key2', str), [])



import datetime
import unittest

from mock import Mock
import pytz

from savory_pie.django import filters
from savory_pie.tests.django import mock_orm
from savory_pie.tests.mock_context import mock_context
from savory_pie.formatters import JSONFormatter


class MockUser(mock_orm.Model):
    pass


now = datetime.datetime.now(tz=pytz.UTC).replace(microsecond=0)
hour = datetime.timedelta(hours=1)

_users = mock_orm.QuerySet(
    MockUser(pk=1, name='alice', age=31, when=now - hour),
    MockUser(pk=2, name='charlie', age=26, when=now),
    MockUser(pk=3, name='bob', age=20, when=now + hour)
)

MockUser.objects.all = Mock(return_value=_users)

_filters = [
    filters.StandardFilter('official_test_user', {'name': 'alice'}),
    filters.StandardFilter('bogus_test_user', {'name': 'Nobody'}),
    filters.StandardFilter('early_name', {'name__lt': 'c00000'}),
    filters.StandardFilter('younger_only', {'age__lt': 25}),
    filters.StandardFilter('older_only', {'age__gt': 25}),
    filters.StandardFilter('alphabetical', {}, order_by=['name']),
    filters.StandardFilter('reverse_alphabetical', {}, order_by=['-name']),
    filters.ParameterizedFilter('name_exact', 'name'),
    filters.ParameterizedFilter('before', 'when__lt'),
    filters.ParameterizedFilter('no_earlier_than', 'when__gte'),
    filters.ParameterizedFilter('names', 'name')
]


class Params():
    def __init__(self, params):
        self.params = params

    def __iter__(self):
        return iter(self.params)

    def get_list(self, name):
        value = self.params[name]
        if isinstance(value, list):
            return value
        else:
            return [value]


class FilterTest(unittest.TestCase):

    def apply_filters(self, filters):
        ctx = mock_context()
        queryset = _users
        params = Params({
            ctx.formatter.convert_to_public_property(name): value
            for name, value
            in filters.items()
        })

        for filter in _filters:
            queryset = filter.filter(ctx, params, queryset)
        return queryset


class StandardFilterTest(FilterTest):

    def test_without_filtering(self):
        results = self.apply_filters({})
        self.assertEqual(3, results.count())
        self.assertEqual(
            ['alice', 'charlie', 'bob'],
            [x.name for x in results]
        )

    def test_good_filter(self):
        results = self.apply_filters({'official_test_user': ''})
        self.assertEqual(1, results.count())
        self.assertEqual(['alice'], [x.name for x in results])

    def test_bad_filter(self):
        results = self.apply_filters({'bogus_test_user': ''})
        self.assertEqual(0, results.count())
        self.assertEqual([], [x.name for x in results])

    def test_multiple_filters(self):
        results = self.apply_filters({'official_test_user': '', 'early_name': ''})
        # older_only: alice,charlie,bob -> alice,charlie
        # early_name: alice,charlie -> alice
        self.assertEqual(1, results.count())
        self.assertEqual(['alice'], [x.name for x in results])

    def test_gt_filter(self):
        results = self.apply_filters({'older_only': ''})
        self.assertEqual(2, results.count())
        self.assertEqual(
            sorted(['alice', 'charlie']),
            sorted([x.name for x in results])
        )

    def test_lt_filter(self):
        results = self.apply_filters({'younger_only': ''})
        self.assertEqual(1, results.count())
        self.assertEqual(['bob'], [x.name for x in results])

    def test_lt_filter_alphabetical(self):
        results = self.apply_filters({'early_name': ''})
        self.assertEqual(2, results.count())
        self.assertEqual(
            sorted(['alice', 'bob']),
            sorted([x.name for x in results])
        )

    def test_ascending_order(self):
        results = self.apply_filters({'alphabetical': ''})
        self.assertEqual(3, results.count())
        self.assertEqual(
            ['alice', 'bob', 'charlie'],
            [x.name for x in results]
        )

    def test_descending_order(self):
        results = self.apply_filters({'reverse_alphabetical': ''})
        self.assertEqual(3, results.count())
        self.assertEqual(
            ['charlie', 'bob', 'alice'],
            [x.name for x in results]
        )

    def test_name_exact_alice(self):
        results = self.apply_filters({'name_exact': 'alice'})
        self.assertEqual(1, results.count())
        self.assertEqual(['alice'], [x.name for x in results])

    def test_name_exact_bob(self):
        results = self.apply_filters({'name_exact': 'bob'})
        self.assertEqual(1, results.count())
        self.assertEqual(['bob'], [x.name for x in results])

    def test_missing_key(self):
        # when presented with invalid filter names, ignore them
        results = self.apply_filters({'foo': '', 'bar': ''})
        self.assertEqual(3, results.count())


class ParameterizedFilterTest(FilterTest):

    def test_name_exact_charlie(self):
        results = self.apply_filters({'name_exact': 'charlie'})
        self.assertEqual(1, results.count())
        self.assertEqual(set(['charlie']), set([x.name for x in results]))

    def test_parameter_data_types(self):
        # get_param_value should assume unparsable data remains a string
        ctx = mock_context()
        ctx.formatter = JSONFormatter()
        foofilter = filters.ParameterizedFilter('foo', 'bar')
        params = Params({'bar': 'unparsable'})
        values = foofilter.get_param_values('bar', ctx, params)
        self.assertEquals(1, len(values))
        self.assertEqual(set(['unparsable']), set(values))
        # parsable data should be parsed as a correct type
        now = datetime.datetime.now(tz=pytz.UTC).replace(microsecond=0)
        for value, svalue in [(11, '11'),
                              (3.14159, '3.14159'),
                              (now, now.isoformat("T"))]:
            params = Params({'bar': svalue})
            othervalues = foofilter.get_param_values('bar', ctx, params)
            self.assertEqual(set([value]), set(othervalues))
            self.assertEqual(type(value), type(othervalues[0]))

    def test_before(self):
        results = self.apply_filters({'before': now.isoformat("T")})
        self.assertEqual(1, results.count())
        self.assertEqual(set(['alice']), set([x.name for x in results]))
        results = self.apply_filters({'before': (now + hour).isoformat("T")})
        self.assertEqual(2, results.count())
        self.assertEqual(set(['alice', 'charlie']), set([x.name for x in results]))
        results = self.apply_filters({'before': (now + 2 * hour).isoformat("T")})
        self.assertEqual(3, results.count())
        self.assertEqual(set(['alice', 'charlie', 'bob']), set([x.name for x in results]))

    def test_no_earlier_than(self):
        results = self.apply_filters({'no_earlier_than': now.isoformat("T")})
        self.assertEqual(2, results.count())
        self.assertEqual(set(['charlie', 'bob']), set([x.name for x in results]))
        results = self.apply_filters({'no_earlier_than': (now + hour).isoformat("T")})
        self.assertEqual(1, results.count())
        self.assertEqual(set(['bob']), set([x.name for x in results]))
        results = self.apply_filters({'no_earlier_than': (now + 2 * hour).isoformat("T")})
        self.assertEqual(0, results.count())

    def test_multiple_names(self):
        results = self.apply_filters({'names': ['charlie', 'bob']})
        self.assertEqual(2, results.count())
        self.assertEqual(set(['charlie', 'bob']), set([x.name for x in results]))



import unittest
from mock import Mock

from savory_pie.auth import (
    authorization,
    authorization_adapter,
    datetime_auth_adapter,
    subobject_auth_adapter,
    uri_auth_adapter
)
from savory_pie.errors import AuthorizationError
from savory_pie.fields import IterableField, URIResourceField, URIListResourceField


class AuthorizationAdapterTestCase(unittest.TestCase):

    def test_adapter(self):
        field = Mock(spec=['to_python_value', '_get', '_compute_property'], name='field')
        field._compute_property.return_value = 'source_key'
        field.to_python_value.side_effect = ['source', 'target']
        source_dict = {'source_key': 'value-source'}
        field._get.return_value = 'target'

        args_name, args_source, args_target = authorization_adapter(field, 'ctx', source_dict, 'target_obj')

        field.to_python_value.called_with('value-source')

        self.assertEqual('source', args_source)
        self.assertEqual('target', args_target)
        self.assertEqual('source_key', args_name)

    def test_authorization_adapter(self):
        field = Mock(spec=['to_python_value', '_get', '_compute_property'], name='field')
        field._compute_property.return_value = 'source_key'
        field.to_python_value.return_value = 'source'
        field._get.return_value = 'target'

        source_dict = {'source_key': 'value-source'}

        name, source, target = datetime_auth_adapter(field, 'ctx', source_dict, 'targetObj')

        field._get.assert_called_with('targetObj')
        field.to_python_value.assert_called_with('ctx', 'value-source')
        self.assertEqual(name, 'source_key')
        self.assertEqual(source, 'source')
        self.assertEqual(target, 'target')

    def test_subobject_auth_adapter_subobject(self):
        field = Mock(spec=['_resource_class', '_compute_property', 'name'], name='fieldName')
        field.name = 'fieldName'
        field._resource_class.return_value = 'FieldResource'
        field._compute_property.return_value = 'source_name'
        source_dict = {'source_name': {'resourceUri': 'uri'}}
        ctx = Mock(spec=['build_resource_uri'])
        ctx.build_resource_uri.return_value = 'target'
        target_obj = Mock(spec=['fieldName'], fieldName='subObject')

        name, source, target = subobject_auth_adapter(field, ctx, source_dict, target_obj)

        ctx.build_resource_uri.assert_called_with('FieldResource')
        field._resource_class.assert_called_with('subObject')
        self.assertEqual(name, 'source_name')
        self.assertEqual(source, 'uri')
        self.assertEqual(target, 'target')


class URIAuthorizationAdapterTestCase(unittest.TestCase):

    def _passthrough_method(self, val):
        return val

    def test_uri_auth_adapter_with_related_manager_field(self):
        field = Mock(spec=IterableField, name='field')
        field.name = 'fieldName'
        field._resource_class.side_effect = self._passthrough_method
        field._compute_property.return_value = 'source_name'
        field.get_iterable.return_value = ['uri1', 'uri3', 'uri2']

        ctx = Mock(spec=['build_resource_uri'])
        ctx.build_resource_uri.side_effect = self._passthrough_method

        target_obj = Mock(spec=['fieldName'])
        target_obj.fieldName = Mock(spec=['all'])

        source_dict = {'source_name': [{'resourceUri': 'uri2'}, {'resourceUri': 'uri1'}]}

        name, source, target = uri_auth_adapter(field, ctx, source_dict, target_obj)
        self.assertEqual(name, 'source_name')
        self.assertEqual(source, ['uri1', 'uri2'])
        self.assertEqual(target, ['uri1', 'uri2', 'uri3'])

    def test_uri_auth_adapter_with_uri_resource_field(self):
        field = Mock(spec=URIResourceField)
        field.name = 'fieldName'
        field._resource_class.side_effect = self._passthrough_method
        field._compute_property.return_value = 'source_name'

        ctx = Mock(spec=['build_resource_uri'])
        ctx.build_resource_uri.side_effect = self._passthrough_method

        target_obj = Mock(spec=['fieldName'])
        target_obj.fieldName = 'uri2'

        source_dict = {'source_name': 'uri1'}

        name, source, target = uri_auth_adapter(field, ctx, source_dict, target_obj)
        self.assertEqual(name, 'source_name')
        self.assertEqual(source, 'uri1')
        self.assertEqual(target, 'uri2')

    def test_uri_auth_adapter_with_uri_list_resource_field(self):
        field = Mock(spec=URIListResourceField)
        field.name = 'fieldName'
        field._resource_class.side_effect = self._passthrough_method
        field._compute_property.return_value = 'source_name'
        field.get_iterable.return_value = ['uri3', 'uri1', 'uri2']

        ctx = Mock(spec=['build_resource_uri'])
        ctx.build_resource_uri.side_effect = self._passthrough_method

        target_obj = Mock(spec=['fieldName'])
        target_obj.fieldName = Mock(spec=['all'])

        source_dict = {'source_name': ['uri2', 'uri1']}

        name, source, target = uri_auth_adapter(field, ctx, source_dict, target_obj)
        self.assertEqual(name, 'source_name')
        self.assertEqual(source, ['uri1', 'uri2'])
        self.assertEqual(target, ['uri1', 'uri2', 'uri3'])


class AuthorizationDecoratorTestCase(unittest.TestCase):

    def test_no_permission(self):
        def adapter(*args):
            self.assertEqual((field, 'ctx', 'source_dict', 'target_object'), args)
            return 'field', 'source', 'target'

        function = Mock()
        auth = authorization(adapter)
        field = Mock(name='field', spec=['permission'])
        field.permission = None
        value = auth(function)
        value(field, 'ctx', 'source_dict', 'target_object')
        function.assert_called_with(field, 'ctx', 'source_dict', 'target_object')

    def test_authorized(self):
        def adapter(*args):
            self.assertEqual((field, 'ctx', 'source_dict', 'target_object'), args)
            return 'field', 'source', 'target'

        function = Mock()
        auth = authorization(adapter)
        field = Mock(name='field', spec=['permission'])
        field.permission.auth_adapter = None
        field.permission.is_write_authorized.return_value = True
        value = auth(function)
        value(field, 'ctx', 'source_dict', 'target_object')
        field.permission.is_write_authorized.assert_called_with('ctx', 'target_object', 'source', 'target')
        function.assert_called_with(field, 'ctx', 'source_dict', 'target_object')

    def test_not_authorized(self):
        def adapter(*args):
            self.assertEqual((field, 'ctx', 'source_dict', 'target_object'), args)
            return 'field', 'source', 'target'

        function = Mock()

        # Raise an exception if it is called.
        function.side_effect = Exception
        auth = authorization(adapter)
        field = Mock(name='field', spec=['permission'])
        field.permission.auth_adapter = None
        field.permission.is_write_authorized.return_value = False
        value = auth(function)

        with self.assertRaises(AuthorizationError):
            value(field, 'ctx', 'source_dict', 'target_object')
            field.permission.is_write_authorized.assert_called_with('ctx', 'target_object', 'source', 'target')



import unittest
import django
from UserDict import UserDict

import mock
from mock import Mock
import django.db.models

from django.core.exceptions import ObjectDoesNotExist

from savory_pie.django import resources, fields
from savory_pie.django.fields import (
    AttributeField,
    AttributeFieldWithModel,
    OneToOneField,
    RelatedManagerField,
    SubModelResourceField,
    URIListResourceField,
    URIResourceField,
    AggregateField, RelatedCountField)
from savory_pie.django.resources import ModelResource, QuerySetResource
from savory_pie.django.utils import Related
from savory_pie.errors import SavoryPieError
from savory_pie.django.validators import ValidationError
from savory_pie.tests.django import mock_orm
from savory_pie.tests.django.mock_request import mock_context


class AttributeFieldTest(unittest.TestCase):
    def test_simple_outgoing(self):
        source_object = Mock()
        source_object.foo = 20

        target_dict = dict()

        field = AttributeField(attribute='foo', type=int)
        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], 20)

    def test_simple_none_outgoing(self):
        source_object = Mock()
        source_object.foo = None

        target_dict = dict()

        field = AttributeField(attribute='foo', type=int)
        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], None)

    def test_multilevel_outgoing(self):
        source_object = Mock()
        source_object.foo.bar = 20

        field = AttributeField(attribute='foo.bar', type=int)

        target_dict = dict()

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['bar'], 20)

    def test_multilevel_incoming(self):
        source_dict = {
            'bar': 20
        }

        field = AttributeField(attribute='foo.bar', type=int)

        target_object = Mock(name='target')

        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(target_object.foo.bar, 20)

    def test_simple_incoming(self):
        source_dict = {
            'foo': 20
        }

        target_object = Mock(name='target')

        field = AttributeField(attribute='foo', type=int)
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(target_object.foo, 20)

    def test_incoming_pushes_and_pops(self):
        source_dict = {
            'foo': 20
        }
        ctx = mock_context()

        target_object = Mock(name='target')

        field = AttributeField(attribute='foo', type=int)
        field.handle_incoming(ctx, source_dict, target_object)

        ctx.assert_has_calls([
            mock.call.push(target_object),
            mock.call.pop(),
        ])

    def test_save(self):
        target_object = Mock(name='target')

        field = AttributeField(attribute='foo', type=int)
        field.save(target_object)

        self.assertTrue(target_object.save.called)

    def test_incoming_read_only(self):
        source_dict = {
            'foo': 20
        }

        target_object = Mock(name='target', spec=[])

        field = AttributeField(attribute='foo', type=int, read_only=True)
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertFalse(hasattr(target_object, 'foo'))

    def test_simple_none_incoming(self):
        source_dict = {
            'foo': None
        }

        target_object = Mock(name='target')

        field = AttributeField(attribute='foo', type=int)
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(target_object.foo, None)

    def test_simple_missing_incoming(self):
        source_dict = {}

        target_object = Mock(name='target')

        field = AttributeField(attribute='foo', type=int)
        with self.assertRaises(ValidationError):
            field.handle_incoming(mock_context(), source_dict, target_object)

    def test_alternate_name_outgoing(self):
        source_object = Mock()
        source_object.foo.bar = 20

        field = AttributeField(attribute='foo.bar', type=int, published_property='foo')

        target_dict = dict()

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], 20)

    def test_alternate_name_incoming(self):
        source_dict = {
            'foo': 20
        }

        field = AttributeField(attribute='foo.bar', type=int, published_property='foo')

        target_object = Mock(name='target')

        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(target_object.foo.bar, 20)

    def test_published_property_camel_case(self):
        source_object = Mock()
        source_object.foo.bar = 20

        field = AttributeField(attribute='foo.bar', type=int, published_property='foo_bar')

        target_dict = dict()

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['fooBar'], 20)

    def test_prepare(self):
        field = AttributeField(attribute='foo.bar.baz', type=int)

        related = Related()
        field.prepare(mock_context(), related)

        self.assertEqual(related._select, {
            'foo__bar'
        })

    def test_prepare_with_use_prefetch(self):
        field = AttributeField(attribute='foo.bar.baz', type=int, use_prefetch=True)

        related = Related()
        field.prepare(mock_context(), related)

        self.assertEqual(related._prefetch, {
            'foo__bar'
        })

    def test_add_field(self):
        field = AttributeField(attribute='foo', type=int)
        source_dict = {'foo': 3}
        filter_args = {}

        field.filter_by_item(mock_context(), filter_args, source_dict)

        self.assertEqual({'foo': 3}, filter_args)


class AttributeFieldWithModelsTest(unittest.TestCase):
    def setUp(self):
        class CharField(Mock):
            pass

        class ForeignKey(Mock):
            pass

        class mock_Model(mock_orm.Model):
            def __getattribute__(self, item):
                result = super(mock_Model, self).__getattribute__(item)
                if item in ['bar', 'baz']:
                    result = result.target
                return result

        class BazModel(mock_Model):
            potato = CharField()
            potato.name = 'potato'
            other = CharField()
            other.name = 'other'
        self.baz = BazModel(potato='idaho')
        BazModel._meta = Mock()
        BazModel._meta.fields = [BazModel.potato, BazModel.other]
        self.BazModel = BazModel

        class BarModel(mock_Model):
            baz = ForeignKey()
            baz.name = 'baz'
            baz.target = self.baz
        my_bar = BarModel()
        BarModel._meta = Mock()
        BarModel._meta.fields = [BarModel.baz]

        class FooModel(mock_Model):
            bar = ForeignKey('bar', BarModel())
            bar.name = 'bar'
            bar.target = my_bar
        FooModel._meta = Mock()
        FooModel._meta.fields = [FooModel.bar]
        self.foo = FooModel()

    def test_simple_incoming(self):
        field = AttributeFieldWithModel(
            attribute='bar.baz.potato',
            type=str,
            model=self.BazModel
        )

        source_dict = {
            'potato': 'abc'
        }
        field.handle_incoming(mock_context(), source_dict, self.foo)
        self.assertEqual(self.baz.potato, 'abc')

    def test_simple_incoming_with_extras(self):
        field = AttributeFieldWithModel(
            attribute='bar.baz.potato',
            type=str,
            model=self.BazModel,
            extras={'other': 'def'}
        )

        source_dict = {
            'potato': 'abc'
        }
        field.handle_incoming(mock_context(), source_dict, self.foo)
        self.assertEqual(self.baz.potato, 'abc')
        self.assertEqual(self.baz.other, 'def')


class URIResourceFieldTest(unittest.TestCase):
    def test_outgoing(self):

        class Resource(ModelResource):
            model_class = Mock()
            parent_resource_path = 'resources'

        field = URIResourceField(attribute='foo', resource_class=Resource)

        source_object = Mock()
        source_object.foo = mock_orm.Model(pk=2)

        target_dict = dict()
        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], 'uri://resources/2')

    def test_incoming(self):

        class Resource(ModelResource):
            model_class = Mock()
            pass

        field = URIResourceField(attribute='foo', resource_class=Resource)

        source_dict = {
            'foo': 'uri://resources/2'
        }
        target_object = Mock()

        related_model = mock_orm.Model(pk=2)

        ctx = mock_context()
        ctx.resolve_resource_uri = Mock(return_value=Resource(related_model))

        field.handle_incoming(ctx, source_dict, target_object)

        ctx.resolve_resource_uri.assert_called_with('uri://resources/2')
        self.assertEqual(target_object.foo, related_model)

    def test_incoming_read_only(self):

        class Resource(ModelResource):
            model_class = Mock()
            pass

        field = URIResourceField(
            attribute='foo',
            resource_class=Resource,
            read_only=True,
        )

        source_dict = {
            'foo': 'uri://resources/2'
        }
        target_object = Mock([])

        related_model = mock_orm.Model(pk=2)

        ctx = mock_context()
        ctx.resolve_resource_uri = Mock(return_value=Resource(related_model))

        field.handle_incoming(ctx, source_dict, target_object)

        self.assertFalse(ctx.resolve_resource_uri.called)
        self.assertFalse(hasattr(target_object, 'foo'))


class SchemaSubfieldFieldTest(unittest.TestCase):
    def test_schema_subfield(self):

        class SubResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='foo', type=int),
            ]

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
                RelatedManagerField(attribute='baz', resource_class=SubResource),
            ]

        resource = resources.SchemaResource(Resource)
        ctx = mock_context()
        ctx.build_resource_uri = lambda resource: 'uri://user/schema/'
        schema = resource.get(ctx)
        self.assertTrue('fields' in schema)
        self.assertTrue('bar' in schema['fields'])
        self.assertTrue('baz' in schema['fields'])
        self.assertTrue('fields' in schema['fields']['baz'])
        self.assertTrue('foo' in schema['fields']['baz']['fields'])


class SubModelResourceFieldTest(unittest.TestCase):
    def test_outgoing(self):

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_object = Mock()
        source_object.foo.bar = 20

        target_dict = dict()

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], {'bar': 20})

    def test_outgoing_with_simple_none(self):

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_object = Mock()
        source_object.foo.bar = None

        target_dict = dict()

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], {'bar': None})

    def test_outgoing_with_submodel_none(self):

        class OtherResource(ModelResource):
            model_class = Mock()
            field = [
                AttributeField(attribute='bar', type=int),
            ]

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                SubModelResourceField(attribute='bar', resource_class=OtherResource),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_object = Mock()
        source_object.foo = None

        target_dict = dict()

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], None)

    def test_incoming_update_existing(self):

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_dict = {
            'foo': {'bar': 20},
        }

        target_object = Mock()
        target_object._meta.get_field().related.field.name = 'bar'
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(target_object.foo.bar, 20)
        target_object.foo.save.assert_called_with()

    @mock.patch('savory_pie.fields.AttributeField._get_object')
    def test_incoming_update_not_dirty(self, get_object):
        target = Mock()
        target.is_dirty.return_value = False
        get_object.return_value = target

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_dict = {
            'foo': {'bar': 20},
        }
        target_object = Mock()
        target_object._meta.get_field().related.field.name = 'bar'
        field.handle_incoming(mock_context(), source_dict, target_object)
        self.assertFalse(target.save.called)

    def test_incoming_with_none_source(self):

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_dict = {}

        target_object = Mock()

        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(target_object.foo, None)

    def test_incoming_with_none_source_with_not_none_existing(self):
        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_dict = {
            'foo': None
        }

        target_object = Mock()
        target_object.foo = UserDict({'bar': 4})
        target_object.foo.pk = 1
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(target_object.foo, None)

    def test_incoming_with_none_subresource(self):

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=Resource)

        source_dict = {
            'foo': {}
        }

        target_object = Mock()
        target_object._meta.get_field().related.field.name = 'bar'
        field.handle_incoming(mock_context(), source_dict, target_object)
        self.assertIsNotNone(target_object.foo)

    def test_incoming_read_only(self):

        class Resource(ModelResource):
            model_class = Mock(spec=[])
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(
            attribute='foo',
            resource_class=Resource,
            read_only=True,
        )

        source_dict = {
            'foo': {'bar': 20},
        }

        target_object = Mock()
        target_object.foo = Mock(['save'])

        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertFalse(hasattr(target_object.foo, 'bar'))
        self.assertFalse(target_object.foo.save.called)

    def test_new_object_incoming(self):

        class MockResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

            @classmethod
            def get_by_source_dict(cls, ctx, sub_source_dict):
                return None

        field = SubModelResourceField(attribute='foo', resource_class=MockResource)

        source_dict = {
            'foo': {'bar': 20},
        }

        # The django ORM makes me sad that this is not a None or AttributeError
        class MockFailsOnFooAccess(Mock):
            def __getattr__(self, name):
                if name == 'foo':
                    raise ObjectDoesNotExist
                else:
                    return super(Mock, self).__getattr__(name)

        target_object = MockFailsOnFooAccess()
        target_object._meta.get_field().related.field.name = 'bar'
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(20, target_object.foo.bar)
        self.assertEqual(MockResource.model_class.return_value, target_object.foo)
        target_object.foo.save.assert_called()

    def test_find_existing_incoming(self):

        mock_model = Mock()

        class MockResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

            @classmethod
            def get_by_source_dict(cls, ctx, sub_source_dict):
                return cls(mock_model)

        field = SubModelResourceField(attribute='foo', resource_class=MockResource)

        source_dict = {
            'foo': {'bar': 20},
        }

        # The django ORM makes me sad that this is not a None or AttributeError
        class MockFailsOnFooAccess(Mock):
            def __getattr__(self, name):
                if name == 'foo':
                    raise ObjectDoesNotExist
                else:
                    return super(Mock, self).__getattr__(name)

        target_object = MockFailsOnFooAccess()
        target_object._meta.get_field().related.field.name = 'bar'
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(20, target_object.foo.bar)
        self.assertEqual(mock_model, target_object.foo)
        target_object.foo.save.assert_called()

    def test_find_existing_by_uri_incoming(self):

        class MockResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=MockResource)

        source_dict = {
            'foo': {
                'resourceUri': '//testsever/api/v1/bar/20',
                'bar': 20
            },
        }

        target_object = Mock()
        target_object._meta.get_field().related.field.name = 'bar'
        ctx = mock_context()
        ctx.resolve_resource_uri = Mock()
        foo_20 = ctx.resolve_resource_uri.return_value = MockResource(Mock())
        field.handle_incoming(ctx, source_dict, target_object)

        ctx.resolve_resource_uri.assert_called_with(source_dict['foo']['resourceUri'])
        target_object.foo.save.assert_called()

        self.assertEqual(foo_20.model, target_object.foo)
        self.assertEqual(20, target_object.foo.bar)

    def test_incoming_with_reverse_foreign_key(self):
        class User(mock_orm.Model):
            name = Mock()

        class UserOwner(mock_orm.Model):
            user = Mock(django.db.models.fields.related.ReverseSingleRelatedObjectDescriptor(Mock()))
            type = Mock()

        class UserOwnerResource(resources.ModelResource):
            model_class = UserOwner

            fields = [
                fields.AttributeField(attribute='type', type=str),
            ]

        class MockResource(ModelResource):
            model_class = User

            owner_field = fields.SubModelResourceField(attribute='owner', resource_class=UserOwnerResource)
            fields = [
                AttributeField(attribute='name', type=str),
                owner_field
            ]

        field = SubModelResourceField(attribute='user', resource_class=MockResource)

        source_dict = {
            'user': {'name': 'username', 'owner': {'type': 'ownertype'}},
        }

        # The django ORM makes me sad that this is not a None or AttributeError
        class MockFailsOnFooAccess(Mock):
            def model(self):
                return Mock()

            def __getattr__(self, name):
                if name == 'user':
                    raise ObjectDoesNotExist
                else:
                    return super(Mock, self).__getattr__(name)

        target_object = MockFailsOnFooAccess()
        target_object._meta.get_field().related.field.name = 'name'
        User.objects.filter().get()._meta.get_field().related.field.name = 'name'

        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual('username', target_object.user.name)
        self.assertEqual('ownertype', target_object.user.owner.type)
        target_object.user.save.assert_called()

    def test_prepare(self):

        class MockResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar.baz', type=int),
            ]

        field = SubModelResourceField(attribute='foo', resource_class=MockResource)

        related = Related()
        field.prepare(mock_context(), related)

        self.assertEqual(related._select, {
            'foo',
            'foo__bar'
        })

    def test_prepare_with_use_prefetch(self):

        class MockResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar.baz', type=int),
            ]

        field = SubModelResourceField(
            attribute='foo',
            resource_class=MockResource,
            use_prefetch=True
        )

        related = Related()
        field.prepare(mock_context(), related)

        self.assertEqual(related._prefetch, {
            'foo',
            'foo__bar'
        })


class OneToOneFieldTest(unittest.TestCase):
    def test_outgoing(self):

        class Resource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = OneToOneField(attribute='foo', resource_class=Resource)

        source_object = Mock()
        source_object.foo.bar = 20

        target_dict = dict()

        field.handle_outgoing(mock_context(), source_object, target_dict)

        self.assertEqual(target_dict['foo'], {'bar': 20})

    def test_new_object_incoming(self):

        class MockResource(ModelResource):
            model_class = Mock()
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = OneToOneField(attribute='foo', resource_class=MockResource)

        source_dict = {
            'foo': {'bar': 20},
        }

        # The django ORM makes me sad that this is not a None or AttributeError
        class MockFailsOnFooAccess(Mock):
            def __getattr__(self, name):
                if name == 'foo':
                    raise ObjectDoesNotExist
                else:
                    return super(Mock, self).__getattr__(name)

        target_object = MockFailsOnFooAccess()
        target_object._meta.get_field().related.field.name = 'bar'
        field.handle_incoming(mock_context(), source_dict, target_object)

        self.assertEqual(20, target_object.foo.bar)
        self.assertEqual(MockResource.model_class.return_value, target_object.foo)
        target_object.foo.save.assert_called()


class RelatedManagerFieldTest(unittest.TestCase):
    def test_outgoing(self):

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = RelatedManagerField(attribute='foo', resource_class=MockResource)

        source_object = mock_orm.Model()
        related_manager = mock_orm.Manager()
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            mock_orm.Model(pk=4, bar=14)
        ))
        source_object.foo = related_manager

        target_dict = {}
        field.handle_outgoing(mock_context(), source_object, target_dict)
        self.assertEqual([{'_id': '4', 'bar': 14}], target_dict['foo'])

    def test_outgoing_with_resource_uri(self):

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            resource_path = 'bar'
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = RelatedManagerField(attribute='foo', resource_class=MockResource)

        source_object = mock_orm.Model()
        related_manager = mock_orm.Manager()
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            mock_orm.Model(pk=4, bar=14)
        ))
        source_object.foo = related_manager

        target_dict = {}

        field.handle_outgoing(mock_context(), source_object, target_dict)
        target = target_dict['foo']
        # Not testing the hash of the dictionary that is tested else were
        self.assertEqual(
            [{'resourceUri': 'uri://bar', 'bar': 14}],
            target
        )

    def test_prepare(self):

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField('bar.baz', type=int)
            ]

        class MockQuerySetResource(QuerySetResource):
            resource_class = MockResource

        field = RelatedManagerField(attribute='foo', resource_class=MockQuerySetResource)

        related = Related()
        field.prepare(mock_context(), related)

        self.assertEqual(related._prefetch, {
            'foo',
            'foo__bar'
        })

    def test_incoming_no_id(self):
        del mock_orm.Model._models[:]

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = RelatedManagerField(attribute='foo', resource_class=MockResource)

        target_obj = mock_orm.Mock()
        related_manager = mock_orm.Manager()
        related_manager.all = Mock(return_value=mock_orm.QuerySet())
        target_obj.foo = related_manager
        source_dict = {
            'foo': [{'bar': 4}],
        }

        model_index = len(mock_orm.Model._models)
        field.handle_incoming(mock_context(), source_dict, target_obj)

        model = mock_orm.Model._models[model_index]
        self.assertEqual(4, model.bar)
        related_manager.add.assert_called_with(model)

    def test_incoming_with_resource_uri(self):
        del mock_orm.Model._models[:]

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = RelatedManagerField(attribute='foo', resource_class=MockResource)

        target_obj = mock_orm.Mock()

        related_manager = mock_orm.Manager()
        related_model = mock_orm.Model(pk=4, bar=10)
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            related_model
        ))
        target_obj.foo = related_manager
        source_dict = {
            'foo': [{
                'resourceUri': '//testsever/api/v2/bar/4',
                'bar': 14
            }],
        }

        model_index = len(mock_orm.Model._models)
        ctx = mock_context()
        ctx.resolve_resource_uri = Mock()
        ctx.resolve_resource_uri.return_value = MockResource(related_model)

        field.handle_incoming(ctx, source_dict, target_obj)
        model = mock_orm.Model._models[model_index - 1]
        self.assertEqual(14, model.bar)

    def test_incoming_delete(self):
        del mock_orm.Model._models[:]

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = RelatedManagerField(attribute='foo', resource_class=MockResource)

        target_obj = mock_orm.Mock()
        related_manager = mock_orm.Manager()
        related_model = mock_orm.Model(pk=4, bar=14)
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            related_model
        ))
        target_obj.foo = related_manager
        source_dict = {
            'foo': [],
        }

        field.handle_incoming(mock_context(), source_dict, target_obj)

        related_manager.remove.assert_called_with(related_model)

    def test_incoming_m2m_add(self):
        """
        Attempting to simulate the following:

        Bar(Model):
            pass

        Foo(Model):
            bars = ManyToManyField(Bar, through='FooBar')

        FooBar(Model):
            bar: ForeignKey(Bar)
            foo: ForeignKey(Foo)

        BarResource(ModelResource):
            model_class = Bar

        FooResource(ModelResource):
            model_class = Foo

            fields = [
                RelatedManagerField('bars', BarResource),
            ]
        """
        class BarResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='some_attribute', type=int),
            ]

        field = RelatedManagerField(attribute='bars', resource_class=BarResource)

        bar = mock_orm.Model(pk=1, someAttribute=1, resourceUri='bar/1')
        foo = mock_orm.Model(pk=2)

        BarResource.key = 'bar'
        BarResource.model = bar

        foobars = mock_orm.QuerySet()
        foobars.create = Mock(return_value=mock_orm.Model(pk=3, foo=foo, bar=bar))

        foobar = mock_orm.Model()
        foobar.objects = foobars

        foo.bars = mock_orm.Manager()
        foo.bars.source_field_name = 'foo'
        foo.bars.target_field_name = 'bar'
        foo.bars.all = Mock(return_value=mock_orm.QuerySet())
        foo.bars.through = foobar
        delattr(foo.bars, 'add')

        source_dict = {
            'bars': [bar.__dict__],
        }

        ctx = mock_context()
        ctx.resolve_resource_uri = Mock(return_value=BarResource)

        field.handle_incoming(ctx, source_dict, foo)

        foobars.create.assert_called_with(foo=foo, bar=bar)

    def test_incoming_m2m_delete(self):
        """
        Attempting to simulate the following:

        Bar(Model):
            pass

        Foo(Model):
            bars = ManyToManyField(Bar, through='FooBar')

        FooBar(Model):
            bar: ForeignKey(Bar)
            foo: ForeignKey(Foo)

        BarResource(ModelResource):
            model_class = Bar

        FooResource(ModelResource):
            model_class = Foo

            fields = [
                RelatedManagerField('bars', BarResource),
            ]
        """
        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='some_attribute', type=int),
            ]

        field = RelatedManagerField(attribute='bars', resource_class=MockResource)

        bar = mock_orm.Model(pk=1)
        foo = mock_orm.Model(pk=2)
        foobar = mock_orm.Model(pk=3, foo=foo, bar=bar)

        foobars = mock_orm.Manager()
        foobars.all = Mock(return_value=mock_orm.QuerySet(foobar))
        foobars.filter = foobars.all
        foobar.objects = foobars

        foo.bars = mock_orm.Manager()
        foo.bars.source_field_name = 'foo'
        foo.bars.target_field_name = 'bar'
        foo.bars.all = Mock(return_value=mock_orm.QuerySet(bar))
        foo.bars.through = foobar
        delattr(foo.bars, 'remove')

        source_dict = {
            'bars': [],
        }

        field.handle_incoming(mock_context(), source_dict, foo)

        foobar.delete.assert_called_with()

    def test_incoming_edit(self):
        del mock_orm.Model._models[:]

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = RelatedManagerField(attribute='foo', resource_class=MockResource)

        target_obj = mock_orm.Mock()
        related_manager = mock_orm.Manager()
        related_model = mock_orm.Model(pk=4, bar=14)
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            related_model
        ))
        target_obj.foo = related_manager
        source_dict = {
            'foo': [{'_id': '4', 'bar': 15}],
        }

        field.handle_incoming(mock_context(), source_dict, target_obj)

        self.assertEqual(15, related_model.bar)
        related_model.save.assert_called()

    def test_incoming_read_only(self):
        del mock_orm.Model._models[:]

        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = RelatedManagerField(
            attribute='foo',
            resource_class=MockResource,
            read_only=True,
        )

        target_obj = mock_orm.Mock()
        related_manager = mock_orm.Manager()
        related_model = mock_orm.Model(pk=4, bar=14)
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            related_model
        ))
        target_obj.foo = related_manager
        source_dict = {
            'foo': [{'_id': '4', 'bar': 15}],
        }

        field.handle_incoming(mock_context(), source_dict, target_obj)

        self.assertEqual(14, related_model.bar)
        self.assertFalse(related_model.save.called)


class URIListResourceFieldTestCase(unittest.TestCase):

    def test_incoming_with_add(self):
        class MockResource(ModelResource):
            key = Mock()
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = URIListResourceField(attribute='foos', resource_class=MockResource)

        source_dict = {
            'foos': ['uri://resources/1', 'uri://resources/2']
        }

        target_object = mock_orm.Mock()
        related_manager = mock_orm.Manager()
        related_manager.all = Mock(return_value=mock_orm.QuerySet())
        target_object.foos = related_manager

        ctx = mock_context()
        foo1_model = Mock()
        foo2_model = Mock()
        mock_resources = Mock()
        resource1 = MockResource(foo1_model)
        resource1.key = 1
        resource2 = MockResource(foo2_model)
        resource2.key = 2
        mock_resources.side_effect = [resource1, resource2]

        ctx.resolve_resource_uri = mock_resources

        field.handle_incoming(ctx, source_dict, target_object)
        related_manager.add.assert_called_with(foo1_model, foo2_model)

    def test_incoming_with_delete(self):
        class MockResource(ModelResource):
            key = Mock()
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = URIListResourceField(attribute='foos', resource_class=MockResource)

        source_dict = {
            'foos': ['uri://resources/1', 'uri://resources/2']
        }

        target_object = mock_orm.Mock()
        related_manager = mock_orm.Manager()
        related_manager.remove = Mock()
        related_model1 = mock_orm.Model(pk=1, bar=11)
        related_model2 = mock_orm.Model(pk=2, bar=12)
        related_model3 = mock_orm.Model(pk=3, bar=13)
        mock_resource1 = MockResource(related_model1)
        mock_resource1.key = 1
        mock_resource2 = MockResource(related_model2)
        mock_resource2.key = 2
        mock_resource3 = MockResource(related_model3)
        mock_resource3.key = 3

        field._resource_class = Mock()
        field._resource_class.side_effect = [mock_resource1, mock_resource2, mock_resource3]
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            related_model1, related_model2, related_model3
        ))
        target_object.foos = related_manager

        ctx = mock_context()
        mock_resources = Mock()
        mock_resources.side_effect = [mock_resource1, mock_resource2]

        ctx.resolve_resource_uri = mock_resources

        field.handle_incoming(ctx, source_dict, target_object)
        related_manager.remove.assert_called_with(related_model3)

    def test_incoming_with_no_change(self):
        class MockResource(ModelResource):
            key = Mock()
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = URIListResourceField(attribute='foos', resource_class=MockResource)

        source_dict = {
            'foos': ['uri://resources/1', 'uri://resources/2']
        }

        target_object = mock_orm.Mock()
        related_manager = mock_orm.Manager()
        related_manager.remove = Mock()
        related_model1 = mock_orm.Model(pk=1, bar=11)
        related_model2 = mock_orm.Model(pk=2, bar=12)
        mock_resource1 = MockResource(related_model1)
        mock_resource1.key = 1
        mock_resource2 = MockResource(related_model2)
        mock_resource2.key = 2

        field._resource_class = Mock()
        field._resource_class.side_effect = [mock_resource1, mock_resource2]
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            related_model1, related_model2
        ))
        target_object.foos = related_manager

        ctx = mock_context()
        mock_resources = Mock()
        mock_resources.side_effect = [mock_resource1, mock_resource2]

        ctx.resolve_resource_uri = mock_resources

        field.handle_incoming(ctx, source_dict, target_object)

        related_manager.remove.assert_called_with()
        related_manager.add.assert_called_with()

    def test_outgoing(self):
        class MockResource(ModelResource):
            model_class = mock_orm.Model
            fields = [
                AttributeField(attribute='bar', type=int),
            ]

        field = URIListResourceField(attribute='foos', resource_class=MockResource)

        source_object = mock_orm.Model()
        related_manager = mock_orm.Manager()
        related_manager.all = Mock(return_value=mock_orm.QuerySet(
            mock_orm.Model(pk=1, bar=14),
            mock_orm.Model(pk=2, bar=14)
        ))

        source_object.foos = related_manager

        ctx = mock_context()
        ctx.build_resource_uri = Mock()
        ctx.build_resource_uri.side_effect = ['uri://resources/1', 'uri://resources/2']

        target_dict = {}
        field.handle_outgoing(ctx, source_object, target_dict)

        self.assertEqual(['uri://resources/1', 'uri://resources/2'], target_dict['foos'])


class StubModel(object):
    objects = None


class TestRelatedCountField(unittest.TestCase):
    @mock.patch('savory_pie.django.fields.AGGREGATE_MAPPER')
    def test_handel_outgoing_count_attr_error(self, mapper):
        ctx = mock_context()

        target_dict = {}
        mapper.get.return_value = sum
        field = RelatedCountField('name')

        count = Mock()
        count.count.return_value = 3
        query_result = Mock()
        query_result.values.return_value = count

        objects = Mock()
        objects.filter.return_value = query_result
        StubModel.objects = objects

        stub_model = StubModel()
        stub_model.id = 7

        field.handle_outgoing(ctx, stub_model, target_dict)

        self.assertEqual(
            3,
            target_dict['name'],
        )
        StubModel.objects.filter.assert_called_with(pk=7)


class TestAggregateField(unittest.TestCase):

    def _get_stub(self, query_resource, id=2):
        query_result = Mock()
        query_result.values.return_value = query_resource

        objects = Mock()
        objects.filter.return_value = query_result
        StubModel.objects = objects

        stub_model = StubModel()
        stub_model.id = id
        return stub_model

    def test_handle_outgoing_sum(self):
        ctx = mock_context()

        mock_model = Mock(name='model')
        mock_model.name__sum = 14
        target_dict = {}

        field = AggregateField('name', django.db.models.Sum)
        field.handle_outgoing(ctx, mock_model, target_dict)

        self.assertEqual(
            14,
            target_dict['name'],
        )

    @mock.patch('savory_pie.django.fields.AGGREGATE_MAPPER')
    def test_raise_unknown_method(self, mapper):
        ctx = mock_context()

        mapper.get.side_effect = AttributeError

        stub_model = self._get_stub([{'name': 23}, {'name': 10}])
        mock_sum = Mock(spec='name')
        mock_sum.name = 'sum'
        target_dict = {}

        field = AggregateField('name', mock_sum)
        with self.assertRaises(AttributeError):
            field.handle_outgoing(ctx, stub_model, target_dict)

    @mock.patch('savory_pie.django.fields.AGGREGATE_MAPPER')
    def test_handel_outgoing_sum_attr_error(self, mapper):
        ctx = mock_context()

        target_dict = {}
        mock_sum = Mock(spec='name')
        mock_sum.name = 'sum'
        mapper.get.return_value = sum
        field = AggregateField('name', mock_sum)

        stub_model = self._get_stub([{'name': 23}, {'name': 10}])
        field.handle_outgoing(ctx, stub_model, target_dict)

        self.assertEqual(
            33,
            target_dict['name'],
        )
        StubModel.objects.filter.assert_called_with(pk=2)

    @mock.patch('savory_pie.django.fields.AGGREGATE_MAPPER')
    def test_handel_outgoing_sum_non(self, mapper):
        ctx = mock_context()

        target_dict = {}
        mock_sum = Mock(spec='name')
        mock_sum.name = 'sum'
        mapper.get.return_value = sum
        field = AggregateField('name', mock_sum)

        stub_model = self._get_stub([{'name': None}])
        field.handle_outgoing(ctx, stub_model, target_dict)

        self.assertEqual(
            0,
            target_dict['name'],
        )

    def test_prepare(self):
        ctx = mock_context()
        ctx.peek.side_effect = IndexError
        related = Mock(name='related')

        field = AggregateField('name.one.two', django.db.models.Count)
        field.prepare(ctx, related)

        related.annotate.assert_called_with(
            django.db.models.Count,
            'name__one__two',
            distinct=True,
        )

    def test_prepare_not_top_level(self):
        ctx = mock_context()
        related = Mock(name='related')

        field = AggregateField('name.one.two', django.db.models.Count)

        with self.assertRaises(SavoryPieError) as cm:
            field.prepare(ctx, related)

        self.assertEqual(
            'RelatedCountField can only be used on a top level ModelResource',
            str(cm.exception)
        )

    def test_handle_outgoing(self):
        ctx = mock_context()

        mock_model = Mock(name='model')
        mock_model.name__count = 14
        target_dict = {}

        field = AggregateField('name', django.db.models.Count)
        field.handle_outgoing(ctx, mock_model, target_dict)

        self.assertEqual(
            14,
            target_dict['name'],
        )



from collections import OrderedDict
from datetime import datetime
from mock import Mock, MagicMock, call, patch
try:
    import ujson as json
except ImportError:
    from warnings import warn
    warn('Using plain JSON instead of uJSON, performance may be degraded.')
    import json
import unittest

from django.contrib.auth.models import User as DjangoUser
from django.http import QueryDict
from savory_pie.django import resources, fields, views
from savory_pie.django.filters import ParameterizedFilter
from savory_pie.tests.django import user_resource_schema, mock_orm, date_str
from savory_pie.tests.mock_context import mock_context
from savory_pie.resources import EmptyParams, _ParamsImpl
from savory_pie.errors import SavoryPieError
from savory_pie import formatters
import django.core.exceptions


class ResourceTest(unittest.TestCase):
    def test_no_allowed_methods(self):
        resource = resources.Resource()
        self.assertEqual(resource.allowed_methods, set())

    def test_allows_get(self):
        resource = resources.Resource()
        resource.get = lambda: dict()
        self.assertEqual(resource.allowed_methods, {'GET'})


class User(mock_orm.Model):
    name = Mock()
    age = Mock()


class UserOwner(mock_orm.Model):
    user = Mock(django.db.models.fields.related.ReverseSingleRelatedObjectDescriptor(Mock()))
    name = Mock()


class AddressableUserResource(resources.ModelResource):
    parent_resource_path = 'users'
    model_class = User

    fields = [
        fields.AttributeField(attribute='name', type=str),
        fields.AttributeField(attribute='age', type=int)
    ]


class UnaddressableUserResource(resources.ModelResource):
    model_class = User

    fields = [
        fields.AttributeField(attribute='name', type=str),
        fields.AttributeField(attribute='age', type=int)
    ]


class SemiUnaddressableUserResource(UnaddressableUserResource):
    parent_resource_path = 'users'


class ComplexUserResource(resources.ModelResource):
    model_class = User

    fields = [
        fields.SubModelResourceField(attribute='manager', resource_class=UnaddressableUserResource),
        fields.RelatedManagerField(attribute='reports', resource_class=UnaddressableUserResource)
    ]


class UserOwnerResource(resources.ModelResource):
    model_class = UserOwner

    fields = [
        fields.AttributeField(attribute='name', type=str),
    ]


class ComplexUserRelationResource(resources.ModelResource):
    model_class = User

    owner_field = fields.SubModelResourceField(attribute='owner', resource_class=UserOwnerResource)

    fields = [
        fields.AttributeField(attribute='name', type=str),
        fields.AttributeField(attribute='age', type=int),
        owner_field,
    ]


class ModelResourceTest(unittest.TestCase):
    def make_request(self, _json, sha=None):
        request = Mock()
        request.META = Mock()
        request.META.get.return_value = sha
        request.read.return_value = _json
        return request

    def test_resource_path(self):
        user = User(pk=1, name='Bob', age=20)
        resource = AddressableUserResource(user)

        self.assertEqual(resource.resource_path, 'users/1')

    @patch('savory_pie.django.resources.dirty_bits')
    def test_dirty_signal_register(self, dirty_bits):
        class NewClazz(mock_orm.Model):
            pass

        class NewClazzResource(resources.ModelResource):
            model_class = NewClazz

        NewClazzResource(NewClazz())
        dirty_bits.register.assert_called_with(NewClazz)

    def test_resource_get_returns_hash(self):
        user = User(pk=1, name='Bob', age=20)

        resource = AddressableUserResource(user)
        dct = resource.get(mock_context(), EmptyParams())

        self.assertEqual(dct, {
            'name': 'Bob',
            'age': 20,
            'resourceUri': 'uri://users/1'
        })

    def test_put(self):
        user = User()

        resource = AddressableUserResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20
        })

        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertTrue(user.save.called)

    def test_complex_put(self):
        user = User()
        user._meta.get_field().related.field.name = 'name'
        user.owner = UserOwner()

        resource = ComplexUserRelationResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20,
            'owner': {'name': 'bob owner'},
        })

        self.assertTrue(user.save.called)
        self.assertTrue(user.owner.save.called)
        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertEqual(user.owner.name, 'bob owner')

    def test_clean_save(self):

        age_field = Mock()
        age_field.name = 'age'
        age_field.value_to_string.return_value = 30

        name_field = Mock()
        name_field.name = 'name'
        name_field.value_to_string.return_value = 'Bob'

        class DirtyUser(mock_orm.Model):
            _fields = [age_field, name_field]
            pk = 3

        class DirtyUserResource(resources.ModelResource):
            parent_resource_path = 'users'
            model_class = DirtyUser

            fields = [
                fields.AttributeField(attribute='name', type=str),
                fields.AttributeField(attribute='age', type=int)
            ]

        dirty_user = DirtyUser()
        dirty_user.save = Mock()
        resource = DirtyUserResource(dirty_user)

        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 30,
            'resourceUri': 'uri://users/1'
        })
        self.assertFalse(dirty_user.save.called)

    def test_dirty_save(self):
        age_field = Mock()
        age_field.name = 'age'
        age_field.value_to_string.return_value = 30

        name_field = Mock()
        name_field.name = 'name'
        name_field.value_to_string.return_value = 'Bob'

        class DirtyUser(mock_orm.Model):
            _fields = [age_field, name_field]
            pk = 3

        class DirtyUserResource(resources.ModelResource):
            parent_resource_path = 'users'
            model_class = DirtyUser

            fields = [
                fields.AttributeField(attribute='name', type=str),
                fields.AttributeField(attribute='age', type=int)
            ]

        dirty_user = DirtyUser()
        dirty_user.save = Mock()
        dirty_user.is_dirty = lambda: True
        resource = DirtyUserResource(dirty_user)

        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 30,
            'resourceUri': 'uri://users/1'
        })
        self.assertTrue(dirty_user.save.called)

    def _dict_compare(self, actual, expected):
        for item1, item2 in zip(expected.items(), actual.items()):
            self.assertEqual(item1, item2, 'Actual not equal to expected {0} {1}'.format(actual, expected))

    def test_qsr_returns_hashes(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        data = resource.get(mock_context(), EmptyParams())

        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 2
        })

        result = data['objects']
        self.assertEqual(len(result), 2)
        first_result = OrderedDict()
        second_result = OrderedDict()

        for key in sorted(result[0].keys()):
            first_result[key] = result[0][key]

        for key in sorted(result[1].keys()):
            second_result[key] = result[1][key]

        alice = OrderedDict({
            '$hash': '01a1b638ddf5318259419587f95ca091a179eeb5',
            'age': 31,
            'name': 'Alice',
            'resourceUri': 'uri://users/1', })
        bob = OrderedDict({
            '$hash': 'df8c8b5694bcd438ea86a87414cf3f59ca42a051',
            'age': 20,
            'name': 'Bob',
            'resourceUri': 'uri://users/2', })

        if first_result['name'] == 'Alice':
            self._dict_compare(first_result, alice)
            self._dict_compare(second_result, bob)
        else:
            self._dict_compare(second_result, alice)
            self._dict_compare(first_result, bob)

    def test_put_with_good_sha(self):
        user = User()
        resource = AddressableUserResource(user)
        resource.get = lambda *args: {"age": 20, "name": "Bob"}
        request = self.make_request('{"age": 20, "name": "Bob"}', 'fd92376f24d6a75974c8da6edf84a834b92ee13c')
        response = views._process_put(mock_context(), resource, request)
        self.assertEqual(response.status_code, 204)

    def test_put_with_bad_sha(self):
        user = User()
        resource = AddressableUserResource(user)
        resource.get = lambda *args: {"age": 20, "name": "Bob"}
        request = self.make_request('{"age": 20, "name": "Bob"}', 'OmManePadmeHumOmManePadmeHumOmManePadmeHum')
        response = views._process_put(mock_context(), resource, request)
        self.assertEqual(response.status_code, 412)

    def test_put_with_missing_required_field(self):
        user = User()
        request = self.make_request('{"name": "Bob"}')   # no age
        response = views._process_put(mock_context(),
                                      AddressableUserResource(user),
                                      request)
        errors = json.loads(response.content)
        self.assertTrue('validation_errors' in errors)
        self.assertEqual(errors['validation_errors'], {'missingField': 'age', 'target': 'User'})

    def test_put_with_foreign_key_none_resource(self):
        user = User()

        resource = ComplexUserRelationResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20,
            'owner': {'name': 'bob owner'},
        })

        self.assertTrue(user.save.called)
        self.assertTrue(user.owner.save.called)
        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertEqual(user.owner.name, 'bob owner')

    def test_put_with_save_false(self):
        user = User()

        resource = AddressableUserResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20
        }, save=False)

        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertFalse(user.save.called)

    def test_delete(self):
        user = User()

        resource = AddressableUserResource(user)
        resource.delete(mock_context())

        self.assertTrue(user.delete.called)

    def test_get_by_source_dict(self):
        source_dict = {
            'name': 'Bob',
            'age': 15,
        }

        with patch.object(AddressableUserResource.model_class, 'objects') as objects:
            AddressableUserResource.get_by_source_dict(mock_context(), source_dict)

        objects.filter.assert_called_with(name='Bob', age=15)

    def test_get_by_source_dict_not_found(self):
        source_dict = {
            'name': 'Bob',
            'age': 15,
        }

        with patch.object(AddressableUserResource.model_class, 'objects') as objects:
            objects.filter.get.return_value = django.core.exceptions.ObjectDoesNotExist
            AddressableUserResource.get_by_source_dict(mock_context(), source_dict)

        objects.filter.assert_called_with(name='Bob', age=15)

    def test_get_by_source_dict_filter_by_item_optional(self):
        source_dict = {
            'name': 'Bob',
            'age': 15,
        }

        with patch.object(ComplexUserResource.model_class, 'objects') as objects:
            ComplexUserResource.get_by_source_dict(mock_context(), source_dict)

        objects.filter.assert_called_with()

    def test_pre_save_optional(self):
        user = Mock(name='user')
        field = Mock(['handle_incoming'])

        class Resource(resources.ModelResource):
            model_class = User
            fields = [field]
        resource = Resource(user)

        resource.put(mock_context(), {'foo': 'bar'})

        user.save.assert_called_with()

    def test_save_fields(self):
        user = Mock(name='user')
        field = Mock(['handle_incoming', 'save'])

        class Resource(resources.ModelResource):
            model_class = User
            fields = [field]
        resource = Resource(user)

        resource.put(mock_context(), {'foo': 'bar'})

        self.assertTrue(field.save.called)

    def test_save_fields_optional(self):
        user = Mock(name='user')
        field = Mock(['handle_incoming'])

        class Resource(resources.ModelResource):
            model_class = User
            fields = [field]
        resource = Resource(user)

        # Tests that an attribute error is not raised
        resource.put(mock_context(), {'foo': 'bar'})


class AddressableUserQuerySetResource(resources.QuerySetResource):
    resource_class = AddressableUserResource


class SemiUnaddressableUserQuerySetResource(resources.QuerySetResource):
    resource_class = SemiUnaddressableUserResource


class FullyUnaddressableUserQuerySetResource(resources.QuerySetResource):
    resource_class = UnaddressableUserResource


class ComplexUserResourceQuerySetResource(resources.QuerySetResource):
    resource_class = ComplexUserResource


class QuerySetResourceTest(unittest.TestCase):
    def remove_hash(self, dct):
        return dict((k, v) for k, v in dct.items() if k[:1] != '$')

    def test_query_set_get(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        data = resource.get(mock_context(), EmptyParams())

        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 2
        })

        self.assertEqual(len(data['objects']), 2)

        data = sorted(data['objects'], key=lambda k: k['name'])

        self.assertEqual(map(self.remove_hash, data), [
            {'resourceUri': 'uri://users/1', 'name': 'Alice', 'age': 31},
            {'resourceUri': 'uri://users/2', 'name': 'Bob', 'age': 20}
        ])

    # def test_query_set_pagination(self):
    #     resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
    #         User(pk=1, name='Alice', age=31),
    #         User(pk=2, name='Bob', age=20),
    #         User(pk=3, name='Zak', age=22),
    #         User(pk=4, name='Dan', age=65)
    #     ))
    #     resource.page_size = 4
    #
    #     data = resource.get(mock_context(), EmptyParams())

    def test_query_set_get_disallow_unfiltered_query(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        resource.allow_unfiltered_query = False
        data = None
        with self.assertRaises(SavoryPieError):
            data = resource.get(mock_context(), EmptyParams())
        self.assertEqual(data, None)  # we should not ever have any data as this is not allowed

    def test_query_set_get_with_valid_filter_param(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        resource.allow_unfiltered_query = False
        resource.filters = [ParameterizedFilter('name', 'name')]
        data = resource.get(mock_context(), _ParamsImpl(QueryDict('name=Alice')))
        self.assertEqual(data['objects'][0]['name'], 'Alice')

    def test_query_set_get_with_invalid_filter_param(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        resource.allow_unfiltered_query = False
        resource.filters = [ParameterizedFilter('name', 'name')]
        data = None
        with self.assertRaises(SavoryPieError):
            data = resource.get(mock_context(), _ParamsImpl(QueryDict('[object Object]=12345')))
        self.assertEqual(data, None)

    def test_get_distinct(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=1, name='Alice', age=31)
        ))
        data = resource.get(mock_context(), EmptyParams())
        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 1
        })

        self.assertEqual(map(self.remove_hash, data['objects']), [
            {'resourceUri': 'uri://users/1', 'name': 'Alice', 'age': 31},
        ])

    def test_empty_queryset(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet())
        data = resource.get(mock_context(), EmptyParams())

        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 0
        })

        self.assertEqual(data['objects'], [])

    def test_addressable_post(self):
        queryset_resource = AddressableUserQuerySetResource()

        new_resource = queryset_resource.post(mock_context(), {
            'name': 'Bob',
            'age': 20
        })

        new_user = new_resource.model
        self.assertEqual(new_resource.resource_path, 'users/' + str(new_user.pk))

        self.assertEqual(new_user.name, 'Bob')
        self.assertEqual(new_user.age, 20)
        self.assertTrue(new_user.save.called)

    def test_semi_unaddressable_post(self):
        queryset_resource = SemiUnaddressableUserQuerySetResource()

        new_resource = queryset_resource.post(mock_context(), {
            'name': 'Bob',
            'age': 20
        })
        self.assertEqual(new_resource.resource_path, 'users/' + str(new_resource.model.pk))

    def test_fully_unaddressable_post(self):
        queryset_resource = FullyUnaddressableUserQuerySetResource()

        new_resource = queryset_resource.post(mock_context(), {
            'name': 'Bob',
            'age': 20
        })
        self.assertIsNone(new_resource.resource_path)

    def test_get_child_resource_success(self):
        alice = User(pk=1, name='Alice', age=31)
        bob = User(pk=2, name='Bob', age=20)

        queryset_resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            alice,
            bob
        ))

        model_resource = queryset_resource.get_child_resource(mock_context(), 1)
        self.assertEqual(model_resource.model, alice)

    def test_get_child_resource_fail(self):
        queryset_resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))

        model_resource = queryset_resource.get_child_resource(mock_context(), 999)
        self.assertIsNone(model_resource)


class ResourcePrepareTest(unittest.TestCase):
    class TestResource(resources.ModelResource):
        model_class = User
        fields = [
            fields.AttributeField(attribute='group.name', type=str),
            fields.AttributeField(attribute='domain.name', type=str)
        ]

    def test_prepare(self):
        related = resources.Related()

        queryset = self.TestResource.prepare(mock_context(), related)
        self.assertEqual(queryset._select, {
            'group',
            'domain'
        })

    def test_prepare_optional(self):
        class NoopField(object):
            def handle_incoming(self, ctx, source_dict, target_obj):
                pass

            def handle_outgoing(self, ctx, source_obj, target_dict):
                pass

        class TestResource(resources.ModelResource):
            model_class = User
            fields = [
                NoopField(),
            ]

        related = resources.Related()
        # Will not raise an error
        TestResource.prepare(mock_context(), related)

    def test_prepere_after_filter(self):
        """
        Django will reset related selects when a filter is added
        """
        queryset = MagicMock()
        queryset_resource = ComplexUserResourceQuerySetResource(queryset)

        queryset_resource.get(mock_context(), EmptyParams())
        calls = call.all().distinct().filter().select_related('manager').prefetch_related('reports').call_list()
        queryset.assert_has_calls(calls)


class DjangoUserResource(resources.ModelResource):
    '''
    Exists to test SchemaResource using Django's User model
    '''
    model_class = DjangoUser
    fields = [
        fields.AttributeField('date_joined', type=datetime),
    ]
    # TODO add filtering and sort order


class SchemaResourceTest(unittest.TestCase):

    def setUp(self):
        self.json_formatter = formatters.JSONFormatter()

    def do_assert_date_equal(self, key):
        field = self.do_get()['fields'][key]
        field['default'] = date_str

        self.assertDictEqual(field, user_resource_schema['fields'][key])

    def do_assert_equal(self, key, assert_type=''):
        getattr(self, 'assert{}Equal'.format(assert_type))(self.do_get()[key], user_resource_schema[key])

    def do_assert_field_equal(self, key):
        self.assertDictEqual(self.do_get()['fields'][key], user_resource_schema['fields'][key])

    def do_get(self):
        """
        Make GET request, return response
        """
        resource = resources.SchemaResource(DjangoUserResource)
        ctx = mock_context()
        ctx.build_resource_uri = lambda resource: 'uri://user/schema/'
        return resource.get(ctx)

    def test_get(self):
        """
        Test basic GET request for a SchemaResource
        """
        self.assertIsInstance(self.do_get(), dict)

    def test_allowed_detail_http_methods(self):
        self.do_assert_equal('allowedDetailHttpMethods', assert_type='List')

    def test_allowed_list_http_methods(self):
        self.do_assert_equal('allowedListHttpMethods', assert_type='List')

    def test_default_format(self):
        self.do_assert_equal('defaultFormat')

    def test_default_limit(self):
        self.do_assert_equal('defaultLimit')

    def test_filtering(self):
        self.do_assert_equal('filtering', assert_type='Dict')

    def test_ordering(self):
        self.do_assert_equal('ordering', assert_type='List')

    def test_resource_uri(self):
        self.do_assert_equal('resourceUri')

    def test_fields_length(self):
        self.assertEqual(len(self.do_get()['fields']), len(user_resource_schema['fields']))

    def test_field_date_joined(self):
        self.do_assert_date_equal('dateJoined')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
from setuptools import find_packages

version = '0.2.1'

setup(
    name='savory-pie',
    version=version,
    url='https://github.com/RueLaLa/savory-pie',
    description='A RESTful api libary with support for django',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'pytz',
        'python-dateutil',
    ],
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



import datetime
from django.db.models import Q


class StandardFilter(object):
    """Filters the results from a query on a :class:`savory_pie.django.resources.QuerySetResource`.
    Each QSR defines a set of available filters for that resource. For a model :class:`Foo` with a
    datetime field *when*, we might define filters like these::

        class FooQuerySetResource(QuerySetResource):
            resource_class = FooResource

            filters = [
                filters.StandardFilter('ancient', {'when__lt': earlyDate}),
                filters.StandardFilter('modern', {'when__gte': earlyDate,
                                                  'when_lt': laterDate}),
                filters.StandardFilter('postmodern', {'when__gte': laterDate}),
                filters.StandardFilter('postmodernReverseChronological',
                                       {'when__gte': laterDate},
                                       order_by=['-when']),
                filters.ParameterizedFilter('limit', 'limit_object_count')
            ]

    The second argument of the StandardFilter constructor (*criteria*) is a dictionary
    of Django-esque `filtering criteria`_.

    .. _`filtering criteria`: https://docs.djangoproject.com/en/dev/topics/db/queries/#retrieving-specific-objects-with-filters

    Filter names should be camel-case and should start with a lowercase letter. They
    can be invoked as part of a web API by appending them to a URL. Here is an example
    of fetching the ten most recent results from the *postmodern* category, and which
    will return a JSON array of information about those ten results::

        curl http://example.com/api/foo?postmodernReverseChronological=&limit=10

    This class is extended by :class:`ParameterizedFilter`, which allows the URL to
    include a parameter (in the example above, the limit on the query size).

    .. warning::

        Use of the special parameter *limit_object_count* will disable
        pagination on if it is under the page size. Use it only for something
        like a top-ten query.

    """

    def __init__(self, name, criteria, order_by=None):
        """
        *name*: A name for invoking the filter in a URL. Should be camel-case with
        a lowercase first letter.

        *criteria*: A dictionary specifying a set of Django-style `filtering criteria`_.

        *order_by*: An optional list of model field names used to sort the query results.
        Preface the fiield name with a minus-sign to reverse the order.

        """
        self.name = name
        self.criteria = criteria or {}
        self._order_by = order_by or []

    def __unicode__(self):
        return u'<' + self.__class__.__name__ + ': ' + self.name + '>'

    def get_param_values(self, name, ctx, params):
        """
        *name*: The name of a parameter which is treated as a key in the *params* QueryDict.

        *ctx*: A context object.

        *params*: A QueryDict of key-value pairs taken from a URL, wherein values are all strings.

        *criteria*: A dictionary to be updated, assuming the value for key *name* can be found
        and successfully parsed in *params*.

        .. note::

            In *StandardFilter* this method does nothing.

        """
        pass

    def is_applicable(self, ctx, params):
        """
        A filter applies in a particular situation if its name (converted to a
        public property by the current formatter) is found among the keys in the
        params._GET querydict. If this filter is applicable, return the converted
        name, otherwise return False.
        """
        name = ctx.formatter.convert_to_public_property(self.name)
        if name in params:
            return name
        else:
            return False

    def build_queryset(self, criteria, queryset):
        if not criteria:
            return queryset

        q = None
        for key, value in criteria.items():
            if q is None:
                q = Q(**{key: value})
            else:
                q |= Q(**{key: value})

        queryset = queryset.filter(q)
        return queryset

    def apply(self, criteria, queryset):
        """
        Apply filtering and sorting criteria to a queryset, return a result queryset.
        """
        # Django just uses 'limit' for this but there might be legitimate uses
        # in models for a field called 'limit', so use a more specific name.
        limit = criteria.pop('limit_object_count', None)

        queryset = self.build_queryset(criteria, queryset)

        if self._order_by is not None:
            queryset = queryset.order_by(*self._order_by)
        if limit:
            limit = limit[0]
            queryset = queryset[:limit]
        return queryset

    def filter(self, ctx, params, queryset):
        """
        Filters (or orders) the queryset according to the specified criteria, if
        this filter is applicable.
        """
        if self.is_applicable(ctx, params):
            queryset = self.apply(self.criteria, queryset)
        return queryset

    def describe(self, ctx, schema_dict):
        """
        Fills in schema_dict with information about the set of valid filtering
        criteria.
        """
        schema_dict['filtering'] = self.criteria
        schema_dict['ordering'] = self._order_by


class ParameterizedFilter(StandardFilter):

    """This extends the :class:`StandardFilter` class, allowing the URL to
    include a parameter.

    """

    def __init__(self, name, paramkey, criteria=None, order_by=None, value_fn=None):
        """
        *name*: A name for invoking the filter in a URL. Should be camel-case with
        a lowercase first letter.

        *paramkey*: The name of a model's field (possibly extended e.g. "fieldname__lt")
        to which the value specified in the URL will be applied.

        .. note::

            There is one special name for *paramkey*, "limit_object_count", which limits
            the query to a specified number of elements. This may break pagination, so **BE CAREFUL**.

        *criteria*: A dictionary specifying a set of Django-style `filtering criteria`_.
        It expects values to be a list.

        *order_by*: An optional list of model field names used to sort the query results.
        Preface the fiield name with a minus-sign to reverse the order.

        *value_fn*: An optional callable which is passed the raw filter value from the
        querystring and which must return the value to be used in the filter.

        """
        self.name = name
        self.paramkey = paramkey
        self.criteria = criteria or {}
        self._order_by = order_by or []
        self.value_fn = value_fn

        self.datatypes = [
            # in order of decreasing specifity/complexity
            datetime.datetime,
            int,   # an int could be mistaken for a float, so try int first
            float,
        ]

    def get_param_values(self, name, ctx, params):
        """
        *name*: The name of a parameter which is treated as a key in the *params* QueryDict.
        The value of the parameter will be parsed as a Python object, and that key-value pair
        will be added to *criteria*.

        *ctx*: A context object.

        *params*: A QueryDict of key-value pairs taken from a URL, wherein values are all strings.

        *criteria*: A dictionary to be updated, assuming the values for key *name* can be found
        and successfully parsed in *params*.

        """
        values = params.get_list(name)

        def apply_value_function(value):
            if self.value_fn is not None:
                value = self.value_fn(value)

            for _type in self.datatypes:
                try:
                    # if a cast doesn't work, a TypeError will be raised
                    # and we'll go on to the next one. if none work, it
                    # remains a string.
                    value = ctx.formatter.to_python_value(_type, value)
                    break
                except TypeError:
                    continue

            return value

        return [apply_value_function(v) for v in values]

    def build_queryset(self, criteria, queryset):
        if not criteria:
            return queryset

        q = None
        for key, values in criteria.items():
            for value in values:
                if q is None:
                    q = Q(**{key: value})
                else:
                    q |= Q(**{key: value})

        queryset = queryset.filter(q)
        return queryset

    def filter(self, ctx, params, queryset):
        """
        Filters (or orders) the queryset according to the specified criteria
        """
        name = self.is_applicable(ctx, params)
        if name:
            criteria = {self.paramkey: self.get_param_values(name, ctx, params)}
            criteria.update(self.criteria)
            queryset = self.apply(criteria, queryset)
        return queryset
